#include "transformable.h"

Transform TransformIdentity() {
    return (Transform){
        .translation = { 0.f, 0.f, 0.f },
        .rotation = QuaternionIdentity(),
        .scale = { 1.f, 1.f, 1.f }
    };
}


Matrix TransformGetMatrix(Transform const *self) {
    Matrix mat = MatrixScale(self->scale.x, self->scale.y, self->scale.z);
    mat = MatrixMultiply(mat, QuaternionToMatrix(self->rotation));
    mat.m12 = self->translation.x;
    mat.m13 = self->translation.y;
    mat.m14 = self->translation.z;
    return mat;
}

Vector3 TransformPosition(Transform const *self, Vector3 local_pos) {
    return Vector3Add(TransformDirection(self, local_pos), self->translation);
}

Vector3 TransformDirection(Transform const *self, Vector3 local_direction) {
    Matrix const m = TransformGetMatrix(self);
    return Vector3Add(Vector3Add(Vector3Scale(MATRIX_RIGHT(m), local_direction.x), Vector3Scale(MATRIX_UP(m), local_direction.y)), Vector3Scale(MATRIX_FORWARD(m), local_direction.z));
}

Vector3 TransformScale(Transform const *self, Vector3 local_scale) {
    return Vector3Multiply(self->scale, TransformDirection(self, local_scale));
}

Quaternion TransformRotation(Transform const *self, Quaternion local_rotation) {
    return QuaternionMultiply(self->rotation, local_rotation);
}

Transform TransformTransform(Transform const *self, Transform const *other) {
    return (Transform) {
        .translation = TransformPosition(self, other->translation),
        .scale = TransformScale(self, other->scale),
        .rotation = TransformRotation(self, other->rotation)
    };
}

Vector3 InverseTransformPosition(Transform const *self, Vector3 global_pos) {
    return Vector3Subtract(InverseTransformDirection(self, global_pos), self->translation);
}

Vector3 InverseTransformDirection(Transform const *self, Vector3 global_direction) {
    Matrix const mat = TransformGetMatrix(self);
    return (Vector3){
        Vector3DotProduct(MATRIX_RIGHT(mat), Vector3Scale(VECTOR3_RIGHT, global_direction.x)),
        Vector3DotProduct(MATRIX_UP(mat), Vector3Scale(VECTOR3_UP, global_direction.y)),
        Vector3DotProduct(MATRIX_FORWARD(mat), Vector3Scale(VECTOR3_FORWARD, global_direction.z))
    };
}

Vector3 InverseTransformScale(Transform const *self, Vector3 global_scale) {
    return Vector3Multiply(Vector3Invert(self->scale), InverseTransformDirection(self, global_scale));
}

Quaternion InverseTransformRotation(Transform const *self, Quaternion global_rotation) {
    return QuaternionMultiply(QuaternionInvert(self->rotation), global_rotation);
}

Transform InverseTransformTransform(Transform const *self, Transform const *other) {
    return (Transform) {
        .translation = InverseTransformPosition(self, other->translation),
        .scale = InverseTransformScale(self, other->scale),
        .rotation = InverseTransformRotation(self, other->rotation)
    };
}
